#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Chars.h"
#include "Print.h"
#include "Sprite.h"

static word PrintS(word vram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        vram = PrintC(vram, c);
        ++p;
    }
    return vram;
}

void PrintStatus() 
{
    // ClearScreen();

    PrintS(VramLeftTop +24 * VramStep, "SCORE");
    PrintS(VramLeftTop +VramRowSize * 3 + 24 * VramStep, "HI-SCORE");
    PrintS(VramLeftTop +VramRowSize * 6 + 24 * VramStep, "STAGE");
    PrintByteNumber2(VramLeftTop +VramRowSize * 6 + 30 * VramStep, CurrentStage + 1);
    PrintScore();
    PrintRemain();
    {
        word vram;
        vram= VramLeftTop +VramRowSize * 12 + 24 * VramStep;
        repeat(12) {
            repeat(8) {
                Put(vram, Char_Map);
                vram += VramStep;
            }
            vram += VramRowSize - 8 * VramStep;
        }
    }
}

void PrintScore()
{
    word vram;
    
    vram = PrintNumber5(VramLeftTop +VramRowSize + 26 * VramStep, Score);
    PrintC(vram, '0');

    vram = PrintNumber5(VramLeftTop +VramRowSize * 4 + 26 * VramStep, HiScore);
    PrintC(vram, '0');
}

void PrintRemain()
{
    word vram;
    byte i;
    vram= VramLeftTop +VramRowSize * 9 + 24 * VramStep;
    if (RemainFighterCount < 5) {
        i = 1;
        while (i < RemainFighterCount) {
            Put2(vram, Char_Remain);
            vram += 2 * VramStep;
            ++i;
        }
    }
    else {
        Put2(vram, Char_Remain);
        vram += 2 * VramStep;
        Erase2(vram);
        vram += 2 * VramStep;
        PrintByteNumber2(VramLeftTop +VramRowSize * 10 + 26 * VramStep, RemainFighterCount - 1);
        i = 3;
    }
    while (i < 4) {
        Erase2(vram);
        vram += 2 * VramStep;
        ++i;
    }
}


static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    word vram;

    vram = VramLeftTop +VramRowSize * 11 + 7 * VramStep;
    repeat (height) {
        repeat (width) {
            vram = PrintC(vram, ' ');
        }
        vram += VramRowSize - width * VramStep;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(VramLeftTop +VramRowSize * 12 + 8 * VramStep, "GAME OVER");
}

static const byte[] TitleBytes = {
    //	R
    0x4c, 0x47, 0x45, 0x4b, 
    0x4c, 0x43, 0x48, 0x4f, 
    0x4c, 0x47, 0x4f, 0x42, 
    0x44, 0x41, 0x44, 0x45, 
    //	U
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
    //	P
    0x4c, 0x47, 0x45, 0x4b, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x47, 0x45, 0x41, 
    0x44, 0x41, 0x40, 0x40, 
    //	T
    0x44, 0x4d, 0x47, 0x41, 
    0x40, 0x4c, 0x43, 0x40, 
    0x40, 0x4c, 0x43, 0x40, 
    0x40, 0x44, 0x41, 0x40, 
    //	U
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
    //	S
    0x48, 0x47, 0x45, 0x4b, 
    0x44, 0x4b, 0x4a, 0x42, 
    0x48, 0x42, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
};

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus();

    {
        ptr<byte> p;
        word vram;
        vram = VramLeftTop +VramRowSize * 8;
        p = TitleBytes;
        repeat (6) {
            Put4(vram, p);
            vram += 4 * VramStep;
            p += 16;
        }
    }
    PrintS(VramLeftTop +VramRowSize * 17 + 5 * VramStep, "PUSH SPACE KEY");
    PrintS(VramLeftTop +VramRowSize * 18 + 5 * VramStep, "OR SHIFT KEY");
    // {
    //     word vram;
    //     byte c;
    //     vram = Vram;
    //     c = Char_Star;
    //     while (c < 150) {
    //         repeat (4) {
    //             Put(vram, c);
    //             ++c;
    //             vram += VramStep;
    //         }
    //         vram += VramRowSize - 4 * VramStep;
    //     }
    // }
}

// void PrintRader(byte x, byte y, byte c)
// {
//     word w, v;
//     word vram;
//     x = (x - 8) >> 4;
//     y = (y - 8) >> 4;
//     y = (y << 2) + y;
//     w = y;
//     w <<= 3;
//     vram = VramLeftTop + VramRowSize * 12 + 24 + w + x;
//     Put(vram, c);
// }